<?php declare(strict_types = 1);
/*
** Zabbix
** Copyright (C) 2024 Cybertrust Japan Co., Ltd.
**
** This program is free software: you can redistribute it and/or modify it under the terms of
** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
**
** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
** See the GNU Affero General Public License for more details.
**
** You should have received a copy of the GNU Affero General Public License along with this program.
** If not, see <https://www.gnu.org/licenses/>.
**/


/**
 * Controller for snapshot creation.
 */
class CControllerSnapshotCreate extends CController {

	protected function checkInput(): bool {
		$fields = [
			'type' => 'required|in '.ZBX_SNAPSHOT_TYPE_HOST.','.ZBX_SNAPSHOT_TYPE_TEMPLATE,
			'hostids' => 'required|not_empty|array_db hosts.hostid'
		];

		return $this->validateInput($fields);
	}

	protected function checkPermissions(): bool {
		switch ($this->getInput('type', ZBX_SNAPSHOT_TYPE_NONE)) {
			case ZBX_SNAPSHOT_TYPE_HOST:
			case ZBX_SNAPSHOT_TYPE_TEMPLATE:
				return $this->checkSnapshotAccess();
			default:
				return false;
		}
	}

	protected function doAction(): void {
		switch ($this->getInput('type', ZBX_SNAPSHOT_TYPE_NONE)) {
			case ZBX_SNAPSHOT_TYPE_HOST:
				$type = ZBX_SNAPSHOT_TYPE_HOST;
				$typestr = 'hosts';
				break;
			case ZBX_SNAPSHOT_TYPE_TEMPLATE:
				$type = ZBX_SNAPSHOT_TYPE_TEMPLATE;
				$typestr = 'templates';
				break;
			default:
				$this->setResponse(new CControllerResponseFatal());
				return;
		}

		$params = [
			'format' => CExportWriterFactory::JSON,
			'prettyprint' => false,
			'options' => [
				$typestr => $this->getInput('hostids', [])
			]
		];

		$result = API::Configuration()->export($params);

		$result_json = json_decode($result, true);
		$now = new DateTime();
		$clock = ($now->format('U'));

		if (array_key_exists($typestr, $result_json['zabbix_export']))
		{
			$members = array_column($result_json['zabbix_export'][$typestr], 'name');

			$db_result = DBexecute(
				'insert into snapshots (name, created_at, type, description, members, data) values (' .
				zbx_dbstr($now->format('Y-m-d\TH:i:sP')) . ',' .
				$clock . ',' .
				$type . ',' .
				"''," .
				zbx_dbstr(json_encode($members)) . ',' .
				zbx_dbstr($result) .
				');'
			);
		}
		else
		{
			$db_result = false;
			error(_('No permissions to referred object or it does not exist!'));
		}

		$output = [];

		if ($db_result) {
			$success = ['title' => _('Snapshot added')];

			if ($messages = get_and_clear_messages()) {
				$success['messages'] = array_column($messages, 'message');
			}

			$output['success'] = $success;
		}
		else {
			$messages = array_column(get_and_clear_messages(), 'message');

			// truncate too long messages to display errors.
			// messages are JSON encoded and then BASE64 encoded.
			// the final BASE64 encoded string should be shorter than 4 KB.
			// so messages should be shorter than roughly 2.5 KB.
			if (strlen(json_encode($messages)) > 2500) {
				$tmp_messages = [];
				$tmp_length = 0;
				foreach ($messages as $m) {
					if (strlen(json_encode($tmp_messages + [$m])) > 2500) {
						$l = strlen(json_encode($tmp_messages));
						$tmp_messages[] = substr($m, 0, 2500 - $l);
						break;
					}

					$tmp_messages[] = $m;
				}

				$messages = $tmp_messages;
			}

			$output['error'] = [
				'title' => _('Cannot add snapshot'),
				'messages' => $messages
			];
		}

		$this->setResponse(new CControllerResponseData(['main_block' => json_encode($output)]));
	}
}
